# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is BesserWisser version 2.0.
#
# The Initial Developer of the Original Code is European Environment
# Agency (EEA).  Portions created by Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Soren Roug, EEA
# Miruna Badescu, Finsiel Romania

# Python imports

# Zope imports
from Products.ZCatalog.ZCatalog import ZCatalog
from AccessControl import ClassSecurityInfo
from DateTime import DateTime

# Product imports
import Resources

class CatalogManager:
    """ This class is responsable with ZCatalog object management.
        The BesserWisser object, holder of all BesserWisser related objects,
        is derived from this class and therefor able to quickly .
    """

    security = ClassSecurityInfo()

    def __init__(self, doCreateCatalog=1):
        """Constructor"""
        self.__catalog = ZCatalog('BesserWisserCatalog')

        #create basic indexes
        self.__createCatalogIndex('PrincipiaSearchSource', 'TextIndex')
        self.__createCatalogIndex('bobobase_modification_time', 'FieldIndex')
        self.__createCatalogIndex('description', 'TextIndex')
        self.__createCatalogIndex('id', 'FieldIndex')
        self.__createCatalogIndex('meta_type', 'FieldIndex')
        self.__createCatalogIndex('path', 'PathIndex')
        self.__createCatalogIndex('title', 'TextIndex')
        #create basic columns
        self.__addColumn('id')
        self.__addColumn('title')
        self.__addColumn('meta_type')
        self.__addColumn('bobobase_modification_time')
        self.__addColumn('summary')

        #extra text indexes
        self.__catalog.addIndex('Question', 'TextIndex')
        self.__catalog.addIndex('Answer', 'TextIndex')
        self.__catalog.addIndex('InquirersName', 'TextIndex')
        self.__catalog.addIndex('LanguagesUnderstood', 'TextIndex')

        #extra field indexes
        self.__catalog.addIndex('CId', 'FieldIndex')
        self.__catalog.addIndex('InquirersEmail', 'FieldIndex')
        self.__catalog.addIndex('Language', 'FieldIndex')
        self.__catalog.addIndex('ExpertId', 'FieldIndex')
        self.__catalog.addIndex('isPrivate', 'FieldIndex')
        self.__catalog.addIndex('askedDateTime', 'FieldIndex')
        self.__catalog.addIndex('answeredDateTime', 'FieldIndex')


    def __clearCatalog(self):
        """Clear Catalog"""
        self.__catalog .manage_catalogClear()

    def __createCatalogIndex(self, p_name, p_type):
        """Create a new index in ZCatalog"""
        try:
            self.__catalog.addIndex(p_name, p_type)
        except:
            pass

    def __deleteCatalogIndex(self, p_name):
        """Delete an index from ZCatalog"""
        try:
            self.__catalog.delIndex(p_name)
        except:
            pass

    def __reindexCatalogIndex(self, p_name, REQUEST):
        """Reindex an index from ZCatalog"""
        self.__catalog.reindexIndex(p_name, REQUEST)

    def __clearCatalogIndex(self, p_name):
        """Clear an index from ZCatalog"""
        self.__catalog.clearIndex(p_name)

    def __addColumn(self, p_name):
        """Create a column"""
        try:
            self.__catalog.addColumn(p_name)
        except:
            pass

    def __delColumn(self, name):
        """Delete a column"""
        try:
            self.__catalog.delColumn(p_name)
        except:
            pass

    def __buildCatalogPath(self, p_item):
        """Creates an id for the item to be added in catalog"""
        return '/' + str(p_item.absolute_url(1))

    def __catalogObject(self, p_object):
        """Catalog an object"""
        self.__catalog.catalog_object(p_object, self.__buildCatalogPath(p_object))

    def __uncatalogObject(self, p_object):
        """Uncatalog an object"""
        self.__catalog.uncatalog_object(self.__buildCatalogPath(p_object))

    def __searchCatalog(self, p_criteria):
        """Search catalog"""
        return self.__catalog(p_criteria)

    def __eliminateDuplicates(self, p_objects):
        """Eliminate duplicates from a list of objects (with ids)"""
        dict = {}
        for l_object in p_objects:
            dict[l_object.id] = l_object
        return dict.values()

    def __getFoundObjects(self, p_list):
        """gets objects from catalog results"""
        return map(self.__catalog.getobject, map(getattr, p_list, ('data_record_id_',)*len(p_list)))

    def __performSearch(self, p_filter):
        """performs a search"""
        return self.__catalog(p_filter)

    def reindexCatalogIndex(self, p_names, REQUEST):
        """Reindex given indexes"""
        for l_name in p_names:
            self.__reindexCatalogIndex(l_name, REQUEST)
        self._p_changed = 1

    def clearCatalogIndex(self, p_names):
        """Clear given indexes"""
        for l_name in p_names:
            self.__clearCatalogIndex(l_name)
        self._p_changed = 1

    def createCatalogIndex(self, p_name, p_type):
        """Create a new index"""
        self.__createCatalogIndex(p_name, p_type)
        self._p_changed = 1

    def deleteCatalogIndex(self, p_names):
        """Delete one or more indexes"""
        for l_name in p_names:
            self.__deleteCatalogIndex(l_name)
        self._p_changed = 1

    def clearCatalog(self):
        """Clear Catalog"""
        self.__clearCatalog()
        self._p_changed = 1

    def getIndexes(self):
        """Return a list with all ZCatalog indexes"""
        return self.__catalog.index_objects()

    def getFullIndex(self):
        """Returns a list with full index (mybrains objects)"""
        return self.__catalog.searchResults()

    def getAllCatalogedObjects(self):
        """Returns a list with all cataloged objects"""
        l_fullindex = self.__catalog.searchResults()
        return map(self.__catalog.getobject, map(getattr, l_fullindex, ('data_record_id_',)*len(l_fullindex)))

    def getCatalogedObjects(self, meta_type=None):
        """Returns a list with all cataloged objects with a specified meta_type"""
        l_results = []
        if meta_type:
            meta_type = self.utConvertToList(meta_type)
            l_results = self.__searchCatalog({'meta_type':meta_type})
            l_results = map(self.__catalog.getobject, map(getattr, l_results, ('data_record_id_',)*len(l_results)))
        else:
            l_results = self.getAllCatalogedObjects()
        return l_results

    def getCatalogObjectPath(self, p_rid):
        """Return the path to a cataloged object given a 'data_record_id_'"""
        return self.__catalog.getpath(p_rid)

    def getCatalogObjectMetadata(self, p_rid):
        """return the correct metadata for the cataloged record id"""
        return self.__catalog.getMetadataForRID(p_rid)

    def getCatalogObjectIndexdata(self, p_rid):
        """return the current index contents for the specific rid"""
        return self.__catalog.getIndexDataForRID(p_rid)

    def catalogObject(self, p_obj):
        """Catalog an object"""
        self.__catalogObject(p_obj)
        self._p_changed = 1

    def uncatalogObject(self, p_obj):
        """Uncatalog an object"""
        self.__uncatalogObject(p_obj)
        self._p_changed = 1

    def recatalogObject(self, p_obj):
        """Recatalog an object"""
        self.__uncatalogObject(p_obj)
        self.__catalogObject(p_obj)
        self._p_changed = 1

    def searchCatalog(self, p_query, p_path):
        """Search Catalog defined indexes"""
        l_result = []
        l_filter = {}
        l_filter['path'] = p_path
        #search in 'title'
        l_criteria = l_filter.copy()
        l_criteria['title'] = p_query
        l_result.extend(self.__searchCatalog(l_criteria))
        #search in 'description'
        l_criteria = l_filter.copy()
        l_criteria['description'] = p_query
        l_result.extend(self.__searchCatalog(l_criteria))
        #get objects
        l_result = map(self.__catalog.getobject, map(getattr, l_result, ('data_record_id_',)*len(l_result)))
        #eliminate duplicates
        return self.__eliminateDuplicates(l_result)

    def _searchAllQuestions(self, p_keyword, p_category, p_expert, p_privacy=0):
        """ """
        l_result = []
        l_result_extend = l_result.extend
        l_filter = {}
        l_filter['meta_type'] = Resources.METATYPE_EXPERT_QUESTION
        if p_category != '':
            l_filter['path'] = p_category
        if p_expert != '':
            l_filter['ExpertId'] = p_expert
        l_filter['isPrivate'] = p_privacy
        #search in 'title'
        l_criteria = l_filter.copy()
        l_criteria['title'] = p_keyword
        l_result_extend(self.__performSearch(l_criteria))
        #search in 'Question'
        l_criteria = l_filter.copy()
        l_criteria['Question'] = p_keyword
        l_result_extend(self.__performSearch(l_criteria))
        #search in 'Answer'
        l_criteria = l_filter.copy()
        l_criteria['Answer'] = p_keyword
        l_result_extend(self.__performSearch(l_criteria))
        #get objects
        l_result = map(self.__catalog.getobject, map(getattr, l_result, ('data_record_id_',)*len(l_result)))
        #eliminate duplicates
        return self.__eliminateDuplicates(l_result)

    security.declarePrivate('getCatalogSearchItems')
    def searchQuestionsInCategory(self, p_keyword, p_path, p_privacy=0):
        """ Search through all questions in a category and subcategories """
        l_result = []
        l_result_extend = l_result.extend
        l_filter = {}
        l_filter['path'] = p_path
        l_filter['meta_type'] = Resources.METATYPE_EXPERT_QUESTION
        l_filter['isPrivate'] = p_privacy
        #search in 'title'
        l_criteria = l_filter.copy()
        l_criteria['title'] = p_keyword
        l_result_extend(self.__performSearch(l_criteria))
        #search in 'Question'
        l_criteria = l_filter.copy()
        l_criteria['Question'] = p_keyword
        l_result_extend(self.__performSearch(l_criteria))
        #search in 'Answer'
        l_criteria = l_filter.copy()
        l_criteria['Answer'] = p_keyword
        l_result_extend(self.__performSearch(l_criteria))
        # get item objects eliminating the duplicates
        return self.__eliminateDuplicates(self.__getFoundObjects(l_result))

    security.declarePrivate('getCatalogSearchItems')
    def getCatalogSearchItems(self, p_keyword):
        """ Free text search for items """
        res = []
        res.extend(self.__performSearch({'title': p_keyword}))
        res.extend(self.__performSearch({'Question': p_keyword}))
        res.extend(self.__performSearch({'Answer': p_keyword}))
        res.extend(self.__performSearch({'InquirersName': p_keyword}))
        res.extend(self.__performSearch({'InquirersEmail': p_keyword}))
        res.extend(self.__performSearch({'Language': p_keyword}))
        res.extend(self.__performSearch({'ExpertId': p_keyword}))
        # get item objects eliminating the duplicates
        return self.__eliminateDuplicates(self.__getFoundObjects(res))

    security.declarePrivate('getListQuestionsForCategory')
    def getListQuestionsForCategory(self, p_category):
        """ Return items on a level - category """
        res = []
        res.extend(self.__performSearch({'title': p_keyword}))
        res.extend(self.__performSearch({'Question': p_keyword}))
        res.extend(self.__performSearch({'Answer': p_keyword}))
        res.extend(self.__performSearch({'InquirersName': p_keyword}))
        res.extend(self.__performSearch({'InquirersEmail': p_keyword}))
        res.extend(self.__performSearch({'Language': p_keyword}))
        res.extend(self.__performSearch({'ExpertId': p_keyword}))
        # get item objects eliminating the duplicates
        return self.__eliminateDuplicates(self.__getFoundObjects(res))

    security.declarePrivate('getAnsweredQuestionsForExpert')
    def getAnsweredQuestionsForExpert(self, p_expertId, 
                                    filterPrivate=None, 
                                    filterAnswered=None,
                                    answeredRangeMin=None):
        """ Return items on a level - category """
        res = [] 
        l_dict = {}
        l_dict['ExpertId'] = p_expertId
        l_dict['meta_type'] = Resources.METATYPE_EXPERT_QUESTION
        if filterPrivate:
            l_dict['isPrivate'] = 0
        if filterAnswered:
            # questions answered in the last answeredRangeMax days
            if answeredRangeMin:
                l_dict['answeredDateTime'] = DateTime() - answeredRangeMin
                l_dict['answeredDateTime_range'] = 'range:min'
            # questions answered ever
            else:
                l_dict['answeredDateTime'] = DateTime('1990/01/02')
                l_dict['answeredDateTime_range'] = 'range:min'
        res.extend(self.__performSearch(l_dict))
        # get item objects eliminating the duplicates
        return self.__eliminateDuplicates(self.__getFoundObjects(res))

    security.declarePrivate('getAskedQuestionsForExpertToday')
    def getAskedQuestionsForExpertToday(self, p_expertId):
        """ questions for an expert in the current day """
        res = []
        l_dict = {'ExpertId': p_expertId}
        l_dict['meta_type'] = Resources.METATYPE_EXPERT_QUESTION
        l_dict['askedDateTime'] = [DateTime(DateTime().Date() + ' 00.00.00'), DateTime(DateTime().Date() + ' 23.59.00')]
        l_dict['askedDateTime_range'] = 'range:min:max'
        res.extend(self.__performSearch(l_dict))
        return self.__eliminateDuplicates(self.__getFoundObjects(res))

    security.declarePrivate('getAskedQuestionsForExpert')
    def getAskedQuestionsForExpert(self, p_expertId):
        """ questions for an expert """
        res = []
        l_dict = {'ExpertId': p_expertId, 'answeredDateTime': None}
        l_dict['meta_type'] = Resources.METATYPE_EXPERT_QUESTION
        res.extend(self.__performSearch(l_dict))
        return self.__eliminateDuplicates(self.__getFoundObjects(res))

    security.declarePrivate('getFilteredQuestions')
    def getFilteredQuestions(self, p_expertId, p_isPrivate):
        """ filtered questions by expert end privacy """
        res = []
        l_dict = {'meta_type':Resources.METATYPE_EXPERT_QUESTION}
        if p_expertId != '':
            l_dict['ExpertId'] = p_expertId
        if p_isPrivate != '' and p_isPrivate != 0:
            l_dict['isPrivate'] = 1
        res.extend(self.__performSearch(l_dict))
        return self.__eliminateDuplicates(self.__getFoundObjects(res))

    security.declarePrivate('getQuestionByCId')
    def getCategoryByCId(self, p_CId):
        """ get a category by CId """
        res = []
        l_dict = {'meta_type': Resources.METATYPE_EXPERT_CATEGORY}
        l_dict['CId'] = p_CId
        res.extend(self.__performSearch(l_dict))
        res = self.__eliminateDuplicates(self.__getFoundObjects(res))
        if len < 1:
            return None
        else:
            return res[0]
